from __future__ import absolute_import

import unittest

import numpy as np
import torch
from nnutils_pytorch import adaptive_maxpool_2d

from torch.nn.functional import adaptive_max_pool2d as torch_adaptive_max_pool2d


class AdaptiveMaxpool2dTest(unittest.TestCase):
    def setUp(self):
        self._s = torch.LongTensor([[3, 4], [2, 8]])

        self._x = torch.Tensor(
            [
                # Img 1 (3 x 4)
                [
                    [1, 2, 3, 4, 99, 99, 99, 99],
                    [5, 6, 7, 8, 99, 99, 99, 99],
                    [9, 10, 11, 12, 99, 99, 99, 99],
                ],
                # Img 2 (2 x 8)
                [
                    [1, 2, 3, 4, 5, 6, 7, 8],
                    [9, 10, 11, 12, 13, 14, 15, 16],
                    [99, 99, 99, 99, 99, 99, 99, 99],
                ],
            ]
        ).resize_(2, 1, 3, 8)

        self._dy = torch.Tensor(
            [
                # Output gradient w.r.t Image 1
                [[3, 6, 9, 12]],
                # Output gradient w.r.t. Image 2
                [[8, 12, 16, 20]],
            ]
        ).resize_(2, 1, 1, 4)

        self._dy_fixed_height = torch.Tensor(
            [
                # Output gradient w.r.t. Image 1
                [[2, 4, 6, 8, 0, 0, 0, 0]],
                # Output gradient w.r.t. Image 2
                [[1, 3, 5, 7, 9, 11, 13, 15]],
            ]
        ).resize_(2, 1, 1, 8)

        self._dy_fixed_width = torch.Tensor(
            [
                # Output gradient w.r.t. Image 1
                [[2, 4], [6, 8], [10, 12]],
                # Output gradient w.r.t. Image 2
                [[1, 3], [5, 7], [0, 0]],
            ]
        ).resize_(2, 1, 3, 2)

        self._expect_y = torch.Tensor(
            [
                # Expected output 1
                [[9, 10, 11, 12]],
                # Expected output 2
                [[10, 12, 14, 16]],
            ]
        ).resize_(2, 1, 1, 4)

        self._expect_y_fixed_height = torch.Tensor(
            [
                # Expected output 1
                [[9, 10, 11, 12, 0, 0, 0, 0]],
                # Expected output 2
                [[9, 10, 11, 12, 13, 14, 15, 16]],
            ]
        ).resize_(2, 1, 1, 8)

        self._expect_y_fixed_width = torch.Tensor(
            [
                # Expected output 1
                [[2, 4], [6, 8], [10, 12]],
                # Expected output 2
                [[4, 8], [12, 16], [0, 0]],
            ]
        ).resize_(2, 1, 3, 2)

        self._expect_dx = torch.Tensor(
            [
                # Input gradient w.r.t. Image 1
                [
                    [0, 0, 0, 0, 0, 0, 0, 0],
                    [0, 0, 0, 0, 0, 0, 0, 0],
                    [3, 6, 9, 12, 0, 0, 0, 0],
                ],
                # Input gradient w.r.t. Image 2
                [
                    [0, 0, 0, 0, 0, 0, 0, 0],
                    [0, 8, 0, 12, 0, 16, 0, 20],
                    [0, 0, 0, 0, 0, 0, 0, 0],
                ],
            ]
        ).resize_(2, 1, 3, 8)

        self._expect_dx_fixed_height = torch.Tensor(
            [
                # Input gradient w.r.t. Image 1
                [
                    [0, 0, 0, 0, 0, 0, 0, 0],
                    [0, 0, 0, 0, 0, 0, 0, 0],
                    [2, 4, 6, 8, 0, 0, 0, 0],
                ],
                # Input gradient w.r.t. Image 2
                [
                    [0, 0, 0, 0, 0, 0, 0, 0],
                    [1, 3, 5, 7, 9, 11, 13, 15],
                    [0, 0, 0, 0, 0, 0, 0, 0],
                ],
            ]
        ).resize_(2, 1, 3, 8)

        self._expect_dx_fixed_width = torch.Tensor(
            [
                # Input gradient w.r.t. Image 1
                [
                    [0, 2, 0, 4, 0, 0, 0, 0],
                    [0, 6, 0, 8, 0, 0, 0, 0],
                    [0, 10, 0, 12, 0, 0, 0, 0],
                ],
                # Input gradient w.r.t. Image 2
                [
                    [0, 0, 0, 1, 0, 0, 0, 3],
                    [0, 0, 0, 5, 0, 0, 0, 7],
                    [0, 0, 0, 0, 0, 0, 0, 0],
                ],
            ]
        ).resize_(2, 1, 3, 8)

    def convert(self, cuda, dtype):
        self._x = self._x.type(dtype)
        self._dy = self._dy.type(dtype)
        self._dy_fixed_height = self._dy_fixed_height.type(dtype)
        self._dy_fixed_width = self._dy_fixed_width.type(dtype)
        self._expect_y = self._expect_y.type(dtype)
        self._expect_y_fixed_height = self._expect_y_fixed_height.type(dtype)
        self._expect_y_fixed_width = self._expect_y_fixed_width.type(dtype)
        self._expect_dx = self._expect_dx.type(dtype)
        self._expect_dx_fixed_height = self._expect_dx_fixed_height.type(dtype)
        self._expect_dx_fixed_width = self._expect_dx_fixed_width.type(dtype)
        if cuda:
            self._x = self._x.cuda()
            self._s = self._s.cuda()
            self._dy = self._dy.cuda()
            self._dy_fixed_height = self._dy_fixed_height.cuda()
            self._dy_fixed_width = self._dy_fixed_width.cuda()
        else:
            self._x = self._x.cpu()
            self._s = self._s.cpu()
            self._dy = self._dy.cpu()
            self._dy_fixed_height = self._dy_fixed_height.cpu()
            self._dy_fixed_width = self._dy_fixed_width.cpu()

    def run_base(self, cuda, ttype):
        self.convert(cuda, ttype)
        x = self._x.detach().requires_grad_()
        xs = self._s.detach()
        y = adaptive_maxpool_2d(x, output_sizes=(1, 4), batch_sizes=xs)
        y.backward(self._dy, retain_graph=True)
        np.testing.assert_array_almost_equal(y.data.cpu(), self._expect_y)
        np.testing.assert_array_almost_equal(x.grad.data.cpu(), self._expect_dx)

    def run_fixed_height(self, cuda, ttype):
        self.convert(cuda, ttype)
        x = self._x.detach().requires_grad_()
        xs = self._s.detach()
        y = adaptive_maxpool_2d(x, output_sizes=(1, None), batch_sizes=xs)
        y.backward(self._dy_fixed_height, retain_graph=True)
        np.testing.assert_array_almost_equal(y.data.cpu(), self._expect_y_fixed_height)
        np.testing.assert_array_almost_equal(
            x.grad.data.cpu(), self._expect_dx_fixed_height
        )

    def run_fixed_width(self, cuda, ttype):
        self.convert(cuda, ttype)
        x = self._x.detach().requires_grad_()
        xs = self._s.detach()
        y = adaptive_maxpool_2d(x, output_sizes=(None, 2), batch_sizes=xs)
        y.backward(self._dy_fixed_width, retain_graph=True)
        np.testing.assert_array_almost_equal(y.data.cpu(), self._expect_y_fixed_width)
        np.testing.assert_array_almost_equal(
            x.grad.data.cpu(), self._expect_dx_fixed_width
        )

    @staticmethod
    def run_compare_reference_smaller_output(cuda, ttype):
        x3 = torch.randn(2, 3, 10, 15).type(ttype)
        xs3 = torch.LongTensor([[4, 5], [8, 6]])
        x1 = x3[0, :, :4, :5].clone().view(1, 3, 4, 5)
        x2 = x3[1, :, :8, :6].clone().view(1, 3, 8, 6)
        if cuda:
            x1 = x1.cuda()
            x2 = x2.cuda()
            x3 = x3.cuda()
            xs3 = xs3.cuda()
        else:
            x1 = x1.cpu()
            x2 = x2.cpu()
            x3 = x3.cpu()
            xs3 = xs3.cpu()
        x1 = x1.requires_grad_()
        x2 = x2.requires_grad_()
        x3 = x3.requires_grad_()
        # Compare forward
        y1 = torch_adaptive_max_pool2d(x1, output_size=(2, 3))
        y2 = torch_adaptive_max_pool2d(x2, output_size=(2, 3))
        y3 = adaptive_maxpool_2d(x3, output_sizes=(2, 3), batch_sizes=xs3)
        np.testing.assert_almost_equal(
            y3.data.cpu().numpy(), torch.cat([y1, y2]).data.cpu().numpy()
        )
        # Compare backward
        dx1, dx2, = torch.autograd.grad(y1.sum() + y2.sum(), [x1, x2])
        dx3, = torch.autograd.grad(y3.sum(), [x3])
        ref = dx3.clone().zero_()
        ref[0, :, :4, :5] = dx1.data
        ref[1, :, :8, :6] = dx2.data
        np.testing.assert_almost_equal(dx3.data.cpu().numpy(), ref.data.cpu().numpy())

    @staticmethod
    def run_compare_reference_larger_output(cuda, ttype):
        x3 = torch.randn(2, 3, 10, 15).type(ttype)
        xs3 = torch.LongTensor([[4, 5], [8, 6]])
        x1 = x3[0, :, :4, :5].clone().view(1, 3, 4, 5)
        x2 = x3[1, :, :8, :6].clone().view(1, 3, 8, 6)
        if cuda:
            x1 = x1.cuda()
            x2 = x2.cuda()
            x3 = x3.cuda()
            xs3 = xs3.cuda()
        else:
            x1 = x1.cpu()
            x2 = x2.cpu()
            x3 = x3.cpu()
            xs3 = xs3.cpu()
        x1 = x1.requires_grad_()
        x2 = x2.requires_grad_()
        x3 = x3.requires_grad_()
        # Compare forward
        y1 = torch_adaptive_max_pool2d(x1, output_size=(10, 15))
        y2 = torch_adaptive_max_pool2d(x2, output_size=(10, 15))
        y3 = adaptive_maxpool_2d(x3, output_sizes=(10, 15), batch_sizes=xs3)
        np.testing.assert_almost_equal(
            y3.data.cpu().numpy(), torch.cat([y1, y2]).data.cpu().numpy()
        )
        # Compare backward
        dx1, dx2, = torch.autograd.grad(y1.sum() + y2.sum(), [x1, x2])
        dx3, = torch.autograd.grad(y3.sum(), [x3])
        ref = dx3.clone().zero_()
        ref[0, :, :4, :5] = dx1.data
        ref[1, :, :8, :6] = dx2.data
        np.testing.assert_almost_equal(dx3.data.cpu().numpy(), ref.data.cpu().numpy())


# Register tests for different types, and different devices.
types = [("torch.FloatTensor", "f32"), ("torch.DoubleTensor", "f64")]
devices = [("cpu", False)]
if torch.cuda.is_available():
    devices += [("gpu", True)]

for ttype, dtype in types:
    for device, use_cuda in devices:
        setattr(
            AdaptiveMaxpool2dTest,
            "test_%s_%s" % (device, dtype),
            lambda self: self.run_base(use_cuda, ttype),
        )
        setattr(
            AdaptiveMaxpool2dTest,
            "test_fixed_height_%s_%s" % (device, dtype),
            lambda self: self.run_fixed_height(use_cuda, ttype),
        )
        setattr(
            AdaptiveMaxpool2dTest,
            "test_fixed_width_%s_%s" % (device, dtype),
            lambda self: self.run_fixed_width(use_cuda, ttype),
        )
        setattr(
            AdaptiveMaxpool2dTest,
            "test_compare_to_reference_smaller_output_%s_%s" % (device, dtype),
            lambda self: self.run_compare_reference_smaller_output(use_cuda, ttype),
        )
        setattr(
            AdaptiveMaxpool2dTest,
            "test_compare_to_reference_larger_output_%s_%s" % (device, dtype),
            lambda self: self.run_compare_reference_larger_output(use_cuda, ttype),
        )


if __name__ == "__main__":
    unittest.main()



import torch

import nnutils_pytorch._C as _nnutils_pytorch


class _AdaptiveAvgpool2d(torch.autograd.Function):
    @classmethod
    def forward(cls, ctx, batch_input, batch_sizes, out_h, out_w):
        assert out_h is not None or out_w is not None
        assert batch_input.is_cuda == batch_sizes.is_cuda
        ctx.save_for_backward(batch_sizes)
        batch_input = batch_input.contiguous()
        batch_sizes = batch_sizes.contiguous()
        N, C, inp_h, inp_w = batch_input.size()
        ctx.inp_h = inp_h
        ctx.inp_w = inp_w

        if out_h is None or out_w is None:
            output_sizes = batch_sizes.clone()
            if out_h is not None:
                output_sizes[:, 0] = out_h
            if out_w is not None:
                output_sizes[:, 1] = out_w
            out_h = inp_h if out_h is None else out_h
            out_w = inp_w if out_w is None else out_w
        else:
            output_sizes = None

        ctx.output_sizes = output_sizes
        batch_output = batch_input.new(N, C, out_h, out_w).zero_()
        _nnutils_pytorch.adaptive_avgpool_2d_fwd(
            x=batch_input, y=batch_output, xs=batch_sizes, ys=output_sizes
        )
        return batch_output

    @classmethod
    def backward(cls, ctx, grad_output):
        batch_sizes, = ctx.saved_tensors
        assert grad_output.is_cuda == batch_sizes.is_cuda
        grad_output = grad_output.contiguous()
        N, C, _, _ = grad_output.size()
        grad_input = grad_output.data.new(N, C, ctx.inp_h, ctx.inp_w).zero_()
        _nnutils_pytorch.adaptive_avgpool_2d_bwd(
            grad_y=grad_output, grad_x=grad_input, xs=batch_sizes, ys=ctx.output_sizes
        )
        return grad_input, None, None, None


class _AdaptiveMaxpool2d(torch.autograd.Function):
    @classmethod
    def forward(cls, ctx, batch_input, batch_sizes, out_h, out_w):
        assert out_h is not None or out_w is not None
        assert batch_input.is_cuda == batch_sizes.is_cuda
        batch_input = batch_input.contiguous()
        batch_sizes = batch_sizes.contiguous()
        N, C, inp_h, inp_w = batch_input.size()
        ctx.inp_h = inp_h
        ctx.inp_w = inp_w
        ctx.out_h = out_h
        ctx.out_w = out_w
        if out_h is None or out_w is None:
            output_sizes = batch_sizes.clone()
            if out_h is not None:
                output_sizes[:, 0] = out_h
            if out_w is not None:
                output_sizes[:, 1] = out_w
            out_h = inp_h if out_h is None else out_h
            out_w = inp_w if out_w is None else out_w
        else:
            output_sizes = None

        batch_output = batch_input.new(N, C, out_h, out_w).zero_()
        index = batch_sizes.new(N, C, out_h, out_w)
        _nnutils_pytorch.adaptive_maxpool_2d_fwd(
            x=batch_input, y=batch_output, index=index, xs=batch_sizes, ys=output_sizes
        )
        ctx.output_sizes = output_sizes
        ctx.save_for_backward(index)
        return batch_output, index

    @classmethod
    def backward(cls, ctx, grad_output, unused_grad_indexes):
        index, = ctx.saved_tensors
        assert grad_output.is_cuda == index.is_cuda
        assert grad_output.size() == index.size()
        grad_output = grad_output.contiguous()
        N, C, _, _ = grad_output.size()
        grad_input = grad_output.new(N, C, ctx.inp_h, ctx.inp_w).zero_()
        _nnutils_pytorch.adaptive_maxpool_2d_bwd(
            grad_y=grad_output, grad_x=grad_input, index=index, ys=ctx.output_sizes
        )
        return grad_input, None, None, None


class _MaskImageFromSizeFunction(torch.autograd.Function):
    @classmethod
    def forward(cls, ctx, batch_input, batch_sizes, mask_value=0, inplace=False):
        ctx.save_for_backward(batch_sizes)
        ctx.mask_value = mask_value
        ctx.inplace = inplace
        batch_output = batch_input if ctx.inplace else batch_input.clone()
        _nnutils_pytorch.mask_image_from_size(
            x=batch_output.contiguous(), xs=batch_sizes.contiguous(), mask=mask_value
        )
        return batch_output

    @classmethod
    def backward(cls, ctx, grad_output):
        batch_sizes, = ctx.saved_tensors
        grad_input = grad_output if ctx.inplace else grad_output.clone()
        _nnutils_pytorch.mask_image_from_size(
            x=grad_input.contiguous(), xs=batch_sizes.contiguous(), mask=0
        )
        return grad_input, None, None, None


def adaptive_avgpool_2d(batch_input, output_sizes, batch_sizes=None):
    """Applies a 2D adaptive average pooling over an input signal composed of
    several input planes.
    You may specify a single dimension for pooling, in that case the other
    dimension will keep its original size.
    If your input is composed of multiple padded images with different sizes,
    you can do the pooling taking into account the original size of each image
    in the batch, by using the batch_sizes argument.
    Args:
        output_size: the target output size (single integer or
            double-integer tuple). One of the two integers may be None, to
            keep the original size in that dimension.
        batch_sizes: a N x 2 matrix containing the size of each image in the
            batch. Default: ``None``
    """
    if batch_sizes is None:
        return torch.nn.functional.adaptive_avg_pool2d(batch_input, output_sizes)
    else:
        if isinstance(output_sizes, (list, tuple)):
            out_h, out_w = output_sizes
        else:
            out_h, out_w = output_sizes, output_sizes
        return _AdaptiveAvgpool2d.apply(batch_input, batch_sizes, out_h, out_w)


def adaptive_maxpool_2d(
    batch_input, output_sizes, batch_sizes=None, return_indices=False
):
    """Applies a 2D adaptive max pooling over an input signal composed of
    several input planes.
    You may specify a single dimension for pooling, in that case the other
    dimension will keep its original size.
    If your input is composed of multiple padded images with different sizes,
    you can do the pooling taking into account the original size of each image
    in the batch, by using the batch_sizes argument.
    Args:
        output_size: the target output size (single integer or
            double-integer tuple). One of the two integers may be None, to
            keep the original size in that dimension.
        batch_sizes: a N x 2 matrix containing the size of each image in the
            batch. Default: ``None``
        return_indices: whether to return pooling indices. Default: ``False``
    """
    if batch_sizes is None:
        return torch.nn.functional.adaptive_max_pool2d(
            batch_input, output_sizes, return_indices
        )
    else:
        if isinstance(output_sizes, (list, tuple)):
            out_h, out_w = output_sizes
        else:
            out_h, out_w = output_sizes, output_sizes
        ret = _AdaptiveMaxpool2d.apply(batch_input, batch_sizes, out_h, out_w)
        return ret if return_indices else ret[0]


def mask_image_from_size(batch_input, batch_sizes=None, mask_value=0, inplace=False):
    """Mask a batch of images (a 4D tensor) from each image size.

    Let (h, w) be the height and width of a given image in the batch, then this
    function sets the value of all pixel coordinates with x >= w or y >= h to
    the value given by the argument ``mask_value''.

    Args:
        batch_input: the input batch (a 4D tensor), with layout N x C x H x W
            where N is the number of images in the batch, C is the number of
            channels of each image, H is the maximum height and W is the
            maximum width.
        batch_sizes: a integer matrix containing the height and width of each
            image in the batch (N x 2 matrix). If None, does not perform any
            masking.
        mask_value: value used for the pixels ``outside'' of the image bounding
            box. Default: 0
        inplace: whether to perform the operation inplace or not.
            Default: ``False``
    """
    if batch_sizes is None:
        return batch_input
    else:
        return _MaskImageFromSizeFunction.apply(
            batch_input, batch_sizes, mask_value, inplace
        )



from __future__ import absolute_import

import unittest

import torch
from nnutils_pytorch import mask_image_from_size
from torch.autograd import Variable


class MaskImageFromSizeTest(unittest.TestCase):
    def setUp(self):
        self.x = torch.randn(5, 3, 213, 217)
        self.xs = torch.LongTensor(
            [[29, 29], [213, 217], [213, 15], [15, 217], [97, 141]]
        )
        # Cost for each output pixel
        self.cy = torch.randn(5, 3, 213, 217)

    def convert(self, cuda, dtype):
        self.x = self.x.type(dtype)
        self.cy = self.cy.type(dtype)
        if cuda:
            self.x = self.x.cuda()
            self.xs = self.xs.cuda()
            self.cy = self.cy.cuda()
        else:
            self.x = self.x.cpu()
            self.xs = self.xs.cpu()
            self.cy = self.cy.cpu()

    def run_base(self, cuda, ttype):
        self.convert(cuda, ttype)
        x = self.x.detach().requires_grad_()
        cy = self.cy.detach()
        y = mask_image_from_size(batch_input=x, batch_sizes=self.xs, mask_value=99)
        # Check forward
        for i, (xi, yi, s) in enumerate(zip(self.x, y.data, self.xs)):
            # Check non-masked area
            d = torch.sum(yi[:, : s[0], : s[1]] != xi[:, : s[0], : s[1]])
            self.assertEqual(
                0, d, msg="Sample {} failed in the non-masked area".format(i)
            )
            # Check masked area
            d1 = torch.sum(yi[:, s[0] :, :] != 99) if s[0] < self.x.size(2) else 0
            d2 = torch.sum(yi[:, :, s[1] :] != 99) if s[1] < self.x.size(3) else 0
            self.assertEqual(
                0, d1 + d2, msg="Sample {} failed in the masked area".format(i)
            )
        # Check gradients
        cost = torch.sum(torch.mul(y, cy))
        dx, = torch.autograd.grad(cost, (x,))
        for i, (xi, yi, s) in enumerate(zip(dx.data, self.cy, self.xs)):
            # Check non-masked area
            d = torch.sum(xi[:, : s[0], : s[1]] != yi[:, : s[0], : s[1]])
            self.assertEqual(
                0, d, msg="Sample {} failed in the non-masked area".format(i)
            )
            # Check masked area
            d1 = torch.sum(xi[:, s[0] :, :] != 0) if s[0] < self.x.size(2) else 0
            d2 = torch.sum(xi[:, :, s[1] :] != 0) if s[1] < self.x.size(3) else 0
            self.assertEqual(
                0, d1 + d2, msg="Sample {} failed in the masked area".format(i)
            )


# Register tests for different types, and different devices.
for ttype, dtype in [
    ("torch.ByteTensor", "u8"),
    ("torch.ShortTensor", "s16"),
    ("torch.IntTensor", "s32"),
    ("torch.LongTensor", "s64"),
    ("torch.FloatTensor", "f32"),
    ("torch.DoubleTensor", "f64"),
]:
    setattr(
        MaskImageFromSizeTest,
        "test_cpu_%s" % dtype,
        lambda self: self.run_base(False, ttype),
    )
    if torch.cuda.is_available():
        setattr(
            MaskImageFromSizeTest,
            "test_gpu_%s" % dtype,
            lambda self: self.run_base(True, ttype),
        )

if __name__ == "__main__":
    unittest.main()



import io
import os
import re
import torch
from setuptools import setup, find_packages
from torch.utils.cpp_extension import BuildExtension, CppExtension, CUDAExtension


extra_compile_args = {
    "cxx": ["-std=c++14", "-O3", "-fopenmp"],
    "nvcc": ["-std=c++14", "-O3", "--compiler-options=-fopenmp"],
}

CC = os.getenv("CC", None)
if CC is not None:
    extra_compile_args["nvcc"].append("-ccbin=" + CC)


def get_cuda_compile_archs(nvcc_flags=None):
    """Get the target CUDA architectures from CUDA_ARCH_LIST env variable"""
    if nvcc_flags is None:
        nvcc_flags = []

    CUDA_ARCH_LIST = os.getenv("CUDA_ARCH_LIST", None)
    if CUDA_ARCH_LIST is not None:
        for arch in CUDA_ARCH_LIST.split(";"):
            m = re.match(r"^([0-9.]+)(?:\(([0-9.]+)\))?(\+PTX)?$", arch)
            assert m, "Wrong architecture list: %s" % CUDA_ARCH_LIST
            cod_arch = m.group(1).replace(".", "")
            com_arch = m.group(2).replace(".", "") if m.group(2) else cod_arch
            ptx = True if m.group(3) else False
            nvcc_flags.extend(
                ["-gencode", "arch=compute_{},code=sm_{}".format(com_arch, cod_arch)]
            )
            if ptx:
                nvcc_flags.extend(
                    [
                        "-gencode",
                        "arch=compute_{},code=compute_{}".format(com_arch, cod_arch),
                    ]
                )

    return nvcc_flags


def get_requirements():
    req_file = os.path.join(os.path.dirname(__file__), "requirements.txt")
    with io.open(req_file, "r", encoding="utf-8") as f:
        return [line.strip() for line in f]


def get_long_description():
    readme_file = os.path.join(os.path.dirname(__file__), "README.md")
    with io.open(readme_file, "r", encoding="utf-8") as f:
        return f.read()


include_dirs = [os.path.dirname(os.path.realpath(__file__)) + "/src"]

headers = [
    "src/adaptive_avgpool_2d.h",
    "src/adaptive_maxpool_2d.h",
    "src/mask_image_from_size.h",
    "src/nnutils/adaptive_pool.h",
    "src/nnutils/utils.h",
    "src/nnutils/cpu/adaptive_avgpool_2d.h",
    "src/nnutils/cpu/adaptive_maxpool_2d.h",
    "src/nnutils/cpu/mask_image_from_size.h",
]

sources = [
    "src/binding.cc",
    "src/adaptive_avgpool_2d.cc",
    "src/adaptive_maxpool_2d.cc",
    "src/mask_image_from_size.cc",
    "src/cpu/adaptive_avgpool_2d.cc",
    "src/cpu/adaptive_maxpool_2d.cc",
    "src/cpu/mask_image_from_size.cc",
]

if torch.cuda.is_available():
    sources += [
        "src/gpu/adaptive_avgpool_2d.cu",
        "src/gpu/adaptive_maxpool_2d.cu",
        "src/gpu/mask_image_from_size.cu",
    ]

    headers += [
        "src/nnutils/gpu/adaptive_avgpool_2d.h",
        "src/nnutils/gpu/adaptive_maxpool_2d.h",
        "src/nnutils/gpu/mask_image_from_size.h",
    ]
    Extension = CUDAExtension

    extra_compile_args["cxx"].append("-DWITH_CUDA")
    extra_compile_args["nvcc"].append("-DWITH_CUDA")
    extra_compile_args["nvcc"].extend(get_cuda_compile_archs())
else:
    Extension = CppExtension


requirements = get_requirements()
long_description = get_long_description()
setup(
    name="nnutils_pytorch",
    version="1.6.0",
    description="PyTorch bindings of the nnutils library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jpuigcerver/nnutils",
    author="Joan Puigcerver",
    author_email="joapuipe@gmail.com",
    license="MIT",
    packages=find_packages(),
    ext_modules=[
        Extension(
            name="nnutils_pytorch._C",
            sources=sources,
            include_dirs=include_dirs,
            extra_compile_args=extra_compile_args,
        )
    ],
    cmdclass={"build_ext": BuildExtension},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Scientific/Engineering :: Image Recognition",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    setup_requires=requirements,
    install_requires=requirements,
)
